using System;
using System.Collections;
using System.Collections.Generic;

namespace Dashboard.Controls
{
    public abstract class ContainerControl : Control, ICollection<Control>
    {
        private readonly List<Control> children = new List<Control>();

        public int Count => children.Count;

        public bool IsReadOnly => false;

        public event EventHandler<ChildEventArgs>? ChildAdded;
        public event EventHandler<ChildEventArgs>? ChildRemoved;

        public void Add(Control item)
        {
            children.Add(item);
            OnChildAdded(new ChildEventArgs(item));
        }

        public void Clear()
        {
            foreach (Control child in this)
            {
                OnChildRemoved(new ChildEventArgs(child));
            }

            children.Clear();
        }

        public bool Contains(Control item)
        {
            return children.Contains(item);
        }

        public void CopyTo(Control[] array, int arrayIndex)
        {
            children.CopyTo(array, arrayIndex);
        }

        public IEnumerator<Control> GetEnumerator()
        {
            return children.GetEnumerator();
        }

        public bool Remove(Control item)
        {
            if (children.Remove(item))
            {
                OnChildRemoved(new ChildEventArgs(item));
                return true;
            }

            return false;
        }

        public virtual void OnChildAdded(ChildEventArgs ea)
        {
            Adopt(ea.Child, this);
            ChildAdded?.Invoke(this, ea);
        }

        public virtual void OnChildRemoved(ChildEventArgs ea)
        {
            Adopt(ea.Child, null);
            ChildRemoved?.Invoke(this, ea);
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return children.GetEnumerator();
        }
    }
    public class ChildEventArgs : EventArgs
    {
        public Control Child { get; }

        public ChildEventArgs(Control child)
        {
            Child = child;
        }
    }
}